#!/usr/bin/env python
import base64
import json
import logging
import os
import sys
from os.path import dirname
from subprocess import check_output
from threading import Thread

import tornado.web
import tornado.websocket
from autobahn.twisted.websocket import WebSocketClientFactory, \
    WebSocketClientProtocol
from twisted.internet import reactor, ssl
from twisted.internet.protocol import ReconnectingClientFactory

logger = logging.getLogger("Jarbas_WebChatTerminalv0.1")
logger.addHandler(logging.StreamHandler(sys.stdout))
logger.setLevel("INFO")

ip = check_output([b'hostname', b'--all-ip-addresses']).replace(b" \n", b"")
port = 8686
clients = {}
lang = "en-us"
platform = "JarbasWebChatClientv0.1"


class MainHandler(tornado.web.RequestHandler):
    def get(self):
        self.render('index.html',ip=ip,  port=port)


class StaticFileHandler(tornado.web.RequestHandler):
    def get(self):
        self.render('js/app.js')


class JarbasWebChatClientProtocol(WebSocketClientProtocol):
    chat = None
    config = {"port": 8286}

    def onConnect(self, response):
        logger.info("Server connected: {0}".format(response.peer))

    def onOpen(self):
        logger.info("WebSocket connection open. ")
        self.factory.client = self
        self.factory.status = "connected"
        self.webchat = Thread(target=self.serve_chat)
        self.webchat.setDaemon(True)
        self.webchat.start()

    def serve_chat(self):
        import tornado.options
        WebSocketHandler.server = self
        port = self.config.get("port", 8286)
        cert = self.config.get("cert_file",
                          dirname(dirname(dirname(__file__))) + '/certs/JarbasServer.crt')
        key = self.config.get("key_file",
                         dirname(dirname(dirname(__file__))) + '/certs/JarbasServer.key')

        tornado.options.parse_command_line()

        routes = [
            tornado.web.url(r"/", MainHandler, name="main"),
            tornado.web.url(r"/static/(.*)", tornado.web.StaticFileHandler, {'path': './'}),
            tornado.web.url(r"/ws", WebSocketHandler)
        ]

        settings = {
            "debug": True,
            "template_path": os.path.join(os.path.dirname(__file__), "templates"),
            "static_path": os.path.join(os.path.dirname(__file__), "static"),
        }

        application = tornado.web.Application(routes, **settings)
        if self.config.get("ssl", False):
            httpServer = tornado.httpserver.HTTPServer(application, ssl_options={
                "certfile": cert,
                "keyfile": key
            })
            url = "https://" + str(ip) + ":" + str(port)
        else:
            httpServer = tornado.httpserver.HTTPServer(application)
            url = "http://" + ip.decode("utf-8") + ":" + str(port)

        tornado.options.parse_command_line()

        print("*********************************************************")
        print("*   Access from web browser " + url)
        print("*********************************************************")

        httpServer.listen(port)
        tornado.ioloop.IOLoop.instance().start()

    def onMessage(self, payload, isBinary):
        if not isBinary:
            payload = payload.decode("utf-8")
            msg = json.loads(payload)
            if msg.get("type", "") == "speak":
                utterance = msg["data"]["utterance"]
                logger.info("Output: " + utterance)
                if self.chat is not None:
                    self.chat.write_message(utterance)
            if msg.get("type", "") == "server.complete_intent_failure":
                logger.error("Output: complete_intent_failure")
        else:
            pass

    def onClose(self, wasClean, code, reason):
        logger.info("WebSocket connection closed: {0}".format(reason))


class WebSocketHandler(tornado.websocket.WebSocketHandler):
    peer = "unknown"
    server = None

    def open(self):
        print('Client IP: ' + self.request.remote_ip)
        self.peer = self.request.remote_ip
        clients[self.peer] = self
        self.write_message("Welcome to Jarbas Web Client")
        self.server.chat = self

    def on_message(self, message):
        utterance = message.strip()
        print("Utterance : " + utterance)
        if utterance:
            if utterance == '"mic_on"':
                pass
            else:
                data = {"utterances": [utterance], "lang": lang}
                context = {"source": self.peer, "destinatary":
                           "skills", "client_name": platform, "peer": self.peer}
                msg = {"data": data,
                       "type": "recognizer_loop:utterance",
                       "context": context}
                msg = json.dumps(msg)
                self.server.sendMessage(bytes(msg, "utf-8"), False)

    def on_close(self):
        global clients
        clients.pop(self.peer)


class JarbasWebChatClientFactory(WebSocketClientFactory, ReconnectingClientFactory):
    protocol = JarbasWebChatClientProtocol

    def __init__(self, *args, **kwargs):
        super(JarbasWebChatClientFactory, self).__init__(*args, **kwargs)
        self.status = "disconnected"
        self.client = None

    # websocket handlers
    def clientConnectionFailed(self, connector, reason):
        logger.info("Client connection failed: " + str(reason) + " .. retrying ..")
        self.status = "disconnected"
        self.retry(connector)

    def clientConnectionLost(self, connector, reason):
        logger.info("Client connection lost: " + str(reason) + " .. retrying ..")
        self.status = "disconnected"
        self.retry(connector)


def connect_to_hivemind(host="127.0.0.1",
                        port=5678, name="Standalone WebChat Terminal",
                        api="test_key", useragent=platform):
    authorization = name + ":" + api
    usernamePasswordDecoded = bytes(authorization, "utf-8")
    api = base64.b64encode(usernamePasswordDecoded)
    headers = {'authorization': api}
    address = u"wss://" + host + u":" + str(port)
    factory = JarbasWebChatClientFactory(address, headers=headers,
                                         useragent=useragent)
    factory.protocol = JarbasWebChatClientProtocol
    contextFactory = ssl.ClientContextFactory()
    reactor.connectSSL(host, port, factory, contextFactory)
    reactor.run()


if __name__ == '__main__':
    # TODO parse args
    connect_to_hivemind()



import base64
import json
import logging
import random
import sys
from threading import Thread
from time import sleep

import hclib
from autobahn.twisted.websocket import WebSocketClientFactory, \
    WebSocketClientProtocol
from twisted.internet import reactor, ssl
from twisted.internet.protocol import ReconnectingClientFactory

platform = "JarbasHackChatBridgev0.1"
logger = logging.getLogger(platform)
logger.addHandler(logging.StreamHandler(sys.stdout))
logger.setLevel("INFO")


class JarbasHackChatClientProtocol(WebSocketClientProtocol):
    hackchat = None
    online_users = []
    connector = None
    waiting_messages = []
    extra_delay = 0
    last_sent = ""
    username = "Jarbas_BOT"
    hack_chat_channel = "JarbasAI"

    def send_queued_message(self):
        while True:
            if len(self.waiting_messages):
                message = ""
                for idx, utterance in enumerate(self.waiting_messages):
                    if utterance:
                        message += utterance + "\n"
                    self.waiting_messages[idx] = ""
                    if len(message) >= 180:
                        words = message.split(" ")
                        new = ""
                        for idx, word in enumerate(words):
                            if len(new) < 180:
                                new += word + " "
                                words[idx] = ""
                        words = [w for w in words if w]
                        self.waiting_messages.insert(0, " ".join(words))
                        message = new
                        break

                if self.extra_delay:
                    sleep(self.extra_delay)
                logger.info("Sent: " + message)
                self.connector.send(message)
                self.last_sent = message
                sleep(random.choice([1, 1.2, 1.5, 1.7, 2, 2.2, 3, 2.6]))
                self.waiting_messages = [ut for ut in self.waiting_messages
                                         if ut]
            else:
                sleep(1)

    def start_hack_chat(self):
        self.hackchat = hclib.HackChat(self.on_hack_message, self.username,
                                       self.hack_chat_channel)

    def on_hack_message(self, connector, data):
        # The second parameter (<data>) is the data received.
        self.connector = connector
        self.online_users = connector.onlineUsers
        user = data.get("nick", "")
        if user and user == self.username:
            # dont answer self
            return
        # Checks if we are being limited
        if data["type"] == "warn":
            logger.info(data["warning"])
            self.extra_delay = 5
            # resend failed
            self.waiting_messages.insert(0, self.last_sent)
        # Checks if someone joined the channel.
        elif data["type"] == "online add":
            # Sends a greeting the person joining the channel.
            #
            connector.send("Hello {}".format(user))
        elif data["type"] == "message":
            utterance = data["text"].lower()
            if utterance == "stop":
                self.waiting_messages = []
                connector.send("ok, stopped")
                return
            if "@" + self.username.lower() in utterance:
                utterance = utterance.replace("@" + self.username.lower(), "")
                msg = {"data": {"utterances": [utterance], "lang": "en-us"},
                       "type": "recognizer_loop:utterance",
                       "context": {"source": self.peer, "destinatary":
                           "https_server", "platform": platform,
                                   "hack_chat_nick": user, "user": user,
                                   "target": "hackchat"}}
                msg = json.dumps(msg)
                self.sendMessage(bytes(msg, "utf-8"), False)
        else:
            logger.debug(str(data))

    def onConnect(self, response):
        logger.info("Server connected: {0}".format(response.peer))
        self.factory.client = self
        self.factory.status = "connected"
        self.username = self.factory.username
        self.hack_chat_channel = self.factory.channel
        logger.info("Channel: {0}".format(self.hack_chat_channel))
        logger.info("Username: {0}".format(self.username))

    def onOpen(self):
        logger.info("WebSocket connection open. ")
        self.chat_thread = Thread(target=self.start_hack_chat)
        self.chat_thread.setDaemon(True)
        self.chat_thread.start()

        self.message_thread = Thread(target=self.send_queued_message)
        self.message_thread.setDaemon(True)
        self.message_thread.start()

    def onMessage(self, payload, isBinary):
        if not isBinary:
            payload = payload.decode("utf-8")
            msg = json.loads(payload)
            user = msg.get("context", {}).get("hack_chat_nick", "")
            if user not in self.online_users:
                logger.info("invalid hack chat user: " + user)
                return
            utterance = ""
            if msg.get("type", "") == "speak":
                utterance = msg["data"]["utterance"]
            elif msg.get("type", "") == "server.complete_intent_failure":
                utterance = "i can't answer that yet"

            if utterance:
                utterance = "@{} , ".format(user) + utterance
                self.waiting_messages.append(utterance)
                logger.info("Queued: " + utterance)
        else:
            pass

    def onClose(self, wasClean, code, reason):
        logger.info("WebSocket connection closed: {0}".format(reason))
        if self.hackchat is not None:
            self.hackchat.leave()
        self.message_thread.join()
        self.chat_thread.join()
        self.hackchat = None
        self.online_users = []
        self.connector = None
        self.factory.client = None
        self.factory.status = "disconnected"
        sys.exit()


class JarbasHackChatClientFactory(WebSocketClientFactory, ReconnectingClientFactory):
    protocol = JarbasHackChatClientProtocol
    username = "jarbasLebot"
    channel = "JarbasBotDemoBridge"

    def __init__(self, *args, **kwargs):
        super(JarbasHackChatClientFactory, self).__init__(*args, **kwargs)
        self.status = "disconnected"
        self.client = None

    # websocket handlers
    def clientConnectionFailed(self, connector, reason):
        logger.info("Client connection failed: " + str(reason) + " .. retrying ..")
        self.status = "disconnected"
        self.retry(connector)

    def clientConnectionLost(self, connector, reason):
        logger.info("Client connection lost: " + str(reason) + " .. retrying ..")
        self.status = "disconnected"
        self.retry(connector)


def connect_to_hackchat(channel, username="Jarbas_BOT", host="127.0.0.1",
                        port=5678, name="Standalone HackChat Bridge", api="test_key", useragent=platform):
    authorization = name + ":" + api
    usernamePasswordDecoded = bytes(authorization, "utf-8")
    api = base64.b64encode(usernamePasswordDecoded)
    headers = {'authorization': api}
    address = u"wss://" + host + u":" + str(port)
    factory = JarbasHackChatClientFactory(address, headers=headers,
                                          useragent=useragent)
    factory.channel = channel
    factory.username = username
    factory.protocol = JarbasHackChatClientProtocol
    contextFactory = ssl.ClientContextFactory()
    reactor.connectSSL(host, port, factory, contextFactory)
    reactor.run()


if __name__ == '__main__':
    # TODO arg parse
    connect_to_hackchat("JarbasAI_" + 10 * random.choice(
        "qwertyuiopplkjhgfdsazxcvbnm"))



#!/usr/bin/env python
#-*- coding:utf-8 -*-
# from mycroft.util import create_signal
import logging
import os
import sys
from os.path import expanduser
from subprocess import check_output
from threading import Thread

import tornado.httpserver
import tornado.ioloop
import tornado.options
import tornado.web
import tornado.websocket
from mycroft.configuration import Configuration
from mycroft.messagebus.client.ws import WebsocketClient
from mycroft.messagebus.message import Message
from mycroft.util.log import LOG

logger = logging.getLogger("Jarbas_WebChatClient")
logger.addHandler(logging.StreamHandler(sys.stdout))
logger.setLevel("INFO")

ip = check_output([b'hostname', b'--all-ip-addresses']).replace(b" \n",
                                                                b"").decode(
    "utf-8")
clients = {}
port = 8092
lang = "en-us"
platform = "JarbasWebChatHiveNodev0.1"


class MainHandler(tornado.web.RequestHandler):
    def get(self):
        self.render('index.html',ip=ip,  port=port)


class StaticFileHandler(tornado.web.RequestHandler):
    def get(self):
        self.render('js/app.js')


class WebSocketHandler(tornado.websocket.WebSocketHandler):
    peer = "unknown"

    def create_internal_emitter(self):
        # connect to mycroft internal websocket
        self.emitter = WebsocketClient()
        self.register_internal_messages()
        self.emitter_thread = Thread(target=self.connect_to_internal_emitter)
        self.emitter_thread.setDaemon(True)
        self.emitter_thread.start()

    def register_internal_messages(self):
        # catch all messages
        self.emitter.on('speak', self.handle_speak)

    def connect_to_internal_emitter(self):
        self.emitter.run_forever()

    def open(self):
        LOG.info('Client IP: ' + self.request.remote_ip)
        self.peer = self.request.remote_ip
        clients[self.peer] = self
        self.create_internal_emitter()
        self.write_message("Welcome to Jarbas Web Client")

    def on_message(self, message):
        utterance = message.strip()
        LOG.info("Utterance : " + utterance)
        if utterance:
            if utterance == '"mic_on"':
                create_signal('startListening')
            else:
                data = {"utterances": [utterance], "lang": lang}
                context = {"source": self.peer, "destinatary":
                           "skills", "client_name": platform, "peer": self.peer}
                self.emitter.emit(Message("recognizer_loop:utterance", data, context))

    def handle_speak(self, event):
        if event.context.get("client_name", platform) == platform:
            peer = event.context.get("peer", "")
            if peer == self.peer:
                self.write_message(event.data['utterance'])

    def on_close(self):
        global clients
        self.emitter.remove("speak", self.handle_speak)
        clients.pop(self.peer)


def launch(config=None):
    import tornado.options

    config = config or Configuration.get().get("hivemind", {}).get("webchat_node", {})
    port = config.get("port", 8286)
    ssl = config.get("ssl", False)
    cert = config.get("cert_file", expanduser('~/.mycroft/hivemind/certs/default.crt'))
    key = config.get("key_file", expanduser('~/.mycroft/hivemind/certs/default.key'))

    tornado.options.parse_command_line()

    routes = [
        tornado.web.url(r"/", MainHandler, name="main"),
        tornado.web.url(r"/static/(.*)", tornado.web.StaticFileHandler, {'path': './'}),
        tornado.web.url(r"/ws", WebSocketHandler)
    ]

    settings = {
        "debug": True,
        "template_path": os.path.join(os.path.dirname(__file__), "templates"),
        "static_path": os.path.join(os.path.dirname(__file__), "static"),
    }

    application = tornado.web.Application(routes, **settings)
    if ssl:
        httpServer = tornado.httpserver.HTTPServer(application, ssl_options={
            "certfile": cert,
            "keyfile": key
        })
        url = "https://" + str(ip) + ":" + str(port)
    else:
        httpServer = tornado.httpserver.HTTPServer(application)
        url = "http://" + str(ip) + ":" + str(port)

    tornado.options.parse_command_line()

    print("*********************************************************")
    print("*   Access from web browser " + url)
    print("*********************************************************")

    httpServer.listen(port)
    tornado.ioloop.IOLoop.instance().start()


if __name__ == '__main__':
    launch()


import base64
import json
import logging
import sys
from threading import Thread

from autobahn.twisted.websocket import WebSocketClientFactory, \
    WebSocketClientProtocol
from responsive_voice import ResponsiveVoice
from twisted.internet import reactor, ssl
from twisted.internet.protocol import ReconnectingClientFactory

from jarbas_hive_mind.terminals.speech.listener import RecognizerLoop

logger = logging.getLogger("Standalone_Mycroft_Client")
logger.addHandler(logging.StreamHandler(sys.stdout))
logger.setLevel("INFO")

platform = "JarbasVoiceTerminalv0.1v0.1"


class JarbasVoiceClientProtocol(WebSocketClientProtocol):

    def onConnect(self, response):
        logger.info("Server connected: {0}".format(response.peer))
        self.factory.client = self
        self.factory.status = "connected"

    def onOpen(self):
        logger.info("WebSocket connection open. ")
        self.loop = RecognizerLoop(self.factory.config)
        self.listen = Thread(target=self.start_listening)
        self.listen.setDaemon(True)
        self.listen.start()

    def handle_record_begin(self):
        logger.info("Begin Recording...")

    def handle_record_end(self):
        logger.info("End Recording...")

    def handle_awoken(self):
        """ Forward mycroft.awoken to the messagebus. """
        logger.info("Listener is now Awake: ")

    def handle_wakeword(self, event):
        logger.info("Wakeword Detected: " + event['utterance'])

    def handle_utterance(self, event):
        context = {'client_name': platform, "source": self.peer + ":speech",
                   'destinatary': "https_server"}
        msg = {"data": {"utterances": event['utterances'], "lang": "en-us"},
               "type": "recognizer_loop:utterance",
               "context": context}

        self.send(msg)

    def handle_unknown(self):
        logger.info('mycroft.speech.recognition.unknown')

    def handle_hotword(self, event):
        config = self.factory.config.get("listener", {})
        ww = config.get("wake_word", "hey mycroft")
        suw = config.get("stand_up_word", "wake up")
        if event["hotword"] != ww and event["hotword"] != suw:
            logger.info("Hotword Detected: " + event['hotword'])

    def handle_sleep(self):
        self.loop.sleep()

    def handle_wake_up(self, event):
        self.loop.awaken()

    def handle_mic_mute(self, event):
        self.loop.mute()

    def handle_mic_unmute(self, event):
        self.loop.unmute()

    def handle_audio_start(self, event):
        """
            Mute recognizer loop
        """
        self.loop.mute()

    def handle_audio_end(self, event):
        """
            Request unmute, if more sources has requested the mic to be muted
            it will remain muted.
        """
        self.loop.unmute()  # restore

    def handle_stop(self, event):
        """
            Handler for mycroft.stop, i.e. button press
        """
        self.loop.force_unmute()

    def start_listening(self):
        self.loop.on('recognizer_loop:utterance', self.handle_utterance)
        self.loop.on('recognizer_loop:record_begin', self.handle_record_begin)
        self.loop.on('recognizer_loop:awoken', self.handle_awoken)
        self.loop.on('recognizer_loop:wakeword', self.handle_wakeword)
        self.loop.on('recognizer_loop:hotword', self.handle_hotword)
        self.loop.on('recognizer_loop:record_end', self.handle_record_end)
        self.loop.run()

    def stop_listening(self):
        self.loop.remove_listener('recognizer_loop:utterance', self.handle_utterance)
        self.loop.remove_listener('recognizer_loop:record_begin', self.handle_record_begin)
        self.loop.remove_listener('recognizer_loop:awoken', self.handle_awoken)
        self.loop.remove_listener('recognizer_loop:wakeword', self.handle_wakeword)
        self.loop.remove_listener('recognizer_loop:hotword', self.handle_hotword)
        self.loop.remove_listener('recognizer_loop:record_end', self.handle_record_end)
        self.listen.join(0)

    def onMessage(self, payload, isBinary):
        if not isBinary:
            payload = payload.decode("utf-8")
            msg = json.loads(payload)
            if msg.get("type", "") == "speak":
                utterance = msg["data"]["utterance"]
                logger.info("Output: " + utterance)
                self.factory.engine.say(utterance)
        else:
            pass

    def send(self, msg):
        msg = json.dumps(msg)
        self.sendMessage(bytes(msg, "utf-8"), False)

    def onClose(self, wasClean, code, reason):
        logger.info("WebSocket connection closed: {0}".format(reason))
        self.stop_listening()
        self.factory.client = None
        self.factory.status = "disconnected"


class JarbasVoiceClientFactory(WebSocketClientFactory, ReconnectingClientFactory):
    protocol = JarbasVoiceClientProtocol
    config = {
        "listener": {
            "sample_rate": 16000,
            "channels": 1,
            "record_wake_words": False,
            "record_utterances": False,
            "phoneme_duration": 120,
            "multiplier": 1.0,
            "energy_ratio": 1.5,
            "wake_word": "hey mycroft",
            "stand_up_word": "wake up"
        },
        "hotwords": {
            "hey mycroft": {
                "module": "pocketsphinx",
                "phonemes": "HH EY . M AY K R AO F T",
                "threshold": 1e-90,
                "lang": "en-us"
            },
            "thank you": {
                "module": "pocketsphinx",
                "phonemes": "TH AE NG K . Y UW .",
                "threshold": 1e-1,
                "listen": False,
                "utterance": "thank you",
                "active": True,
                "sound": "",
                "lang": "en-us"
            },
            "wake up": {
                "module": "pocketsphinx",
                "phonemes": "W EY K . AH P",
                "threshold": 1e-20,
                "lang": "en-us"
            }
        },
        "stt": {
            "deepspeech_server": {
                "uri": "http://localhost:8080/stt"
            },
            "kaldi": {
                "uri": "http://localhost:8080/client/dynamic/recognize"
            }
        }
    }

    def __init__(self, *args, **kwargs):
        super(JarbasVoiceClientFactory, self).__init__(*args, **kwargs)
        self.status = "disconnected"
        self.client = None
        # TODO make optional
        self.engine = ResponsiveVoice(gender="female")

    # websocket handlers
    def clientConnectionFailed(self, connector, reason):
        logger.info(
            "Client connection failed: " + str(reason) + " .. retrying ..")
        self.status = "disconnected"
        self.retry(connector)

    def clientConnectionLost(self, connector, reason):
        logger.info(
            "Client connection lost: " + str(reason) + " .. retrying ..")
        self.status = "disconnected"
        self.retry(connector)


def connect_to_hivemind(host="127.0.0.1",
                        port=5678, name="Standalone Voice Terminal",
                        api="test_key", useragent=platform):
    authorization = name + ":" + api
    usernamePasswordDecoded = bytes(authorization, "utf-8")
    api = base64.b64encode(usernamePasswordDecoded)
    headers = {'authorization': api}
    address = u"wss://" + host + u":" + str(port)
    factory = JarbasVoiceClientFactory(address, headers=headers,
                                       useragent=useragent)
    factory.protocol = JarbasVoiceClientProtocol
    contextFactory = ssl.ClientContextFactory()
    reactor.connectSSL(host, port, factory, contextFactory)
    reactor.run()


if __name__ == '__main__':
    # TODO parse args
    connect_to_hivemind()



__author__ = "jarbas"

import os
import base64
import json
from os import makedirs
from os.path import dirname, join, exists, expanduser


def gen_api(user="demo_user", save=False):
    k = os.urandom(32)
    k = base64.urlsafe_b64encode(k)
    k = "JARBAS_"+str(k)
    db_dir = expanduser("~/.mycroft/hivemind/database")
    if not exists(db_dir):
        makedirs(db_dir)
    if not exists(join(db_dir, "users.json")):
        users = {}
    else:
        with open(join(db_dir, "users.json"), "r") as f:
            users = json.load(f)
    while k in users.keys():
        k = gen_api(user)
    k = k[:-1]
    if save:
        users[k] = {"id": user, "last_active": 0, "name": user}
        with open(join(dirname(__file__), "database", "users.json"), "w") as f:
            data = json.dumps(users)
            f.write(data)
    return k


def gen_admin_api(user="admin", save=False):
    k = os.urandom(32)
    k = base64.urlsafe_b64encode(k)
    k = "JARBAS_"+str(k)
    db_dir = expanduser("~/.mycroft/hivemind/database")
    if not exists(db_dir):
        makedirs(db_dir)
    if not exists(join(db_dir, "admins.json")):
        users = {}
    else:
        with open(join(db_dir, "admins.json"), "r") as f:
            users = json.load(f)
    while k in users.keys():
        k = gen_api(user)
    k = k[:-1]
    if save:
        users[k] = {"id": user, "last_active": 0, "name": user}
        with open(join(dirname(__file__), "database", "admins.json"), "w") as f:
            data = json.dumps(users)
            f.write(data)
    return k


if __name__ == "__main__":
    gen_admin_api("jarbas")


import base64
import json
import logging
import sys
from threading import Thread
from time import sleep

from autobahn.twisted.websocket import WebSocketClientFactory, \
    WebSocketClientProtocol
from fbchat import log, Client
from fbchat.utils import Message
from twisted.internet import reactor, ssl
from twisted.internet.protocol import ReconnectingClientFactory

platform = "JarbasFaceBookBridgev0.1"
logger = logging.getLogger(platform)
logger.addHandler(logging.StreamHandler(sys.stdout))
logger.setLevel("INFO")


class FaceBot(Client):
    protocol = None

    def bind(self, protocol):
        self.protocol = protocol

    def onMessage(self, author_id, message_object, thread_id, thread_type, **kwargs):
        self.markAsDelivered(author_id, thread_id)
        self.markAsRead(author_id)

        log.info("{} from {} in {}".format(message_object, thread_id, thread_type.name))

        # If you're not the author, echo
        if author_id != self.uid:
            msg = {"data": {"utterances": [message_object.text],
                            "lang": "en-us"},
                   "type": "recognizer_loop:utterance",
                   "context": {"source": self.protocol.peer,
                               "destinatary": "https_server", "platform": platform,
                               "user": author_id, "fb_chat_id": author_id,
                               "target": "fbchat"}}
            msg = json.dumps(msg)
            self.protocol.clients[author_id] = {"type": thread_type}
            self.protocol.sendMessage(bytes(msg, "utf-8"), False)


class JarbasFacebookClientProtocol(WebSocketClientProtocol):
    facebook = None
    chat_thread = None
    clients = {}
    MAIL = ""
    PASSWD = ""

    def start_fb_chat(self):
        self.facebook = FaceBot(self.MAIL, self.PASSWD)
        self.facebook.listen()

    def onConnect(self, response):
        logger.info("Server connected: {0}".format(response.peer))
        self.factory.client = self
        self.factory.status = "connected"
        self.MAIL = self.factory.mail
        self.PASSWD = self.factory.passwd

    def onOpen(self):
        logger.info("WebSocket connection open. ")

        self.chat_thread = Thread(target=self.start_fb_chat)
        self.chat_thread.setDaemon(True)
        self.chat_thread.start()
        while self.facebook is None:
            sleep(1)
        self.facebook.bind(self)

    def onMessage(self, payload, isBinary):
        if not isBinary:
            payload = payload.decode("utf-8")
            msg = json.loads(payload)
            utterance = ""
            if msg.get("type", "") == "speak":
                utterance = msg["data"]["utterance"]
            elif msg.get("type", "") == "server.complete_intent_failure":
                utterance = "does not compute"

            if utterance:
                user_id = msg["context"]["fb_chat_id"]
                self.facebook.send(Message(text=utterance),
                                   thread_id=user_id,
                                   thread_type=self.clients[user_id]["type"])
        else:
            pass

    def onClose(self, wasClean, code, reason):
        logger.info("WebSocket connection closed: {0}".format(reason))
        self.factory.client = None
        self.factory.status = "disconnected"


class JarbasFacebookClientFactory(WebSocketClientFactory, ReconnectingClientFactory):
    protocol = JarbasFacebookClientProtocol
    name = "facebook bridge"
    mail = "fake@mail.com"
    passwd = "ChangeMe"

    def __init__(self, *args, **kwargs):
        super(JarbasFacebookClientFactory, self).__init__(*args, **kwargs)
        self.status = "disconnected"
        self.client = None

    # websocket handlers
    def clientConnectionFailed(self, connector, reason):
        logger.info("Client connection failed: " + str(reason) + " .. retrying ..")
        self.status = "disconnected"
        self.retry(connector)

    def clientConnectionLost(self, connector, reason):
        logger.info("Client connection lost: " + str(reason) + " .. retrying ..")
        self.status = "disconnected"
        self.retry(connector)


def connect_to_facebook(mail, password, host="127.0.0.1", port=5678,
                        name="facebook bridge", api="test_key", useragent=platform):
    authorization = name + ":" + api
    usernamePasswordDecoded = bytes(authorization, "utf-8")
    api = base64.b64encode(usernamePasswordDecoded)
    headers = {'authorization': api}
    address = u"wss://" + host + u":" + str(port)
    factory = JarbasFacebookClientFactory(address, headers=headers,
                                          useragent=useragent)
    factory.protocol = JarbasFacebookClientProtocol
    contextFactory = ssl.ClientContextFactory()
    reactor.connectSSL(host, port, factory, contextFactory)
    reactor.run()


if __name__ == '__main__':
    # TODO arg parse
    connect_to_facebook("", "")



from OpenSSL import crypto
from socket import gethostname
from os.path import exists, join
from os import makedirs
import random


def create_self_signed_cert(cert_dir, name="jarbas_hivemind"):
    """
    If name.crt and name.key don't exist in cert_dir, create a new
    self-signed cert and key pair and write them into that directory.
    """

    CERT_FILE = name + ".crt"
    KEY_FILE = name + ".key"
    cert_path = join(cert_dir, CERT_FILE)
    key_path = join(cert_dir, KEY_FILE)

    if not exists(join(cert_dir, CERT_FILE)) \
            or not exists(join(cert_dir, KEY_FILE)):
        # create a key pair
        k = crypto.PKey()
        k.generate_key(crypto.TYPE_RSA, 1024)

        # create a self-signed cert
        cert = crypto.X509()
        cert.get_subject().C = "PT"
        cert.get_subject().ST = "Europe"
        cert.get_subject().L = "Mountains"
        cert.get_subject().O = "Jarbas AI"
        cert.get_subject().OU = "Powered by Mycroft-Core"
        cert.get_subject().CN = gethostname()
        cert.set_serial_number(random.randint(0, 2000))
        cert.gmtime_adj_notBefore(0)
        cert.gmtime_adj_notAfter(10 * 365 * 24 * 60 * 60)
        cert.set_issuer(cert.get_subject())
        cert.set_pubkey(k)
        cert.sign(k, 'sha1')
        if not exists(cert_dir):
            makedirs(cert_dir)
        open(cert_path, "wt").write(
            crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
        open(join(cert_dir, KEY_FILE), "wt").write(
            crypto.dump_privatekey(crypto.FILETYPE_PEM, k))

    return cert_path, key_path




import base64
import json
import logging
import sys
from threading import Thread

from autobahn.twisted.websocket import WebSocketClientFactory, \
    WebSocketClientProtocol
from twisted.internet import reactor, ssl
from twisted.internet.protocol import ReconnectingClientFactory

platform = "JarbasCliTerminalv0.1"
logger = logging.getLogger(platform)
logger.addHandler(logging.StreamHandler(sys.stdout))
logger.setLevel("INFO")


class JarbasCliClientProtocol(WebSocketClientProtocol):

    def onConnect(self, response):
        logger.info("Server connected: {0}".format(response.peer))
        self.factory.client = self
        self.factory.status = "connected"

    def onOpen(self):
        logger.info("WebSocket connection open. ")
        self.input_loop = Thread(target=self.get_cli_input)
        self.input_loop.setDaemon(True)
        self.input_loop.start()

    def onMessage(self, payload, isBinary):
        if not isBinary:
            payload = payload.decode("utf-8")
            msg = json.loads(payload)
            if msg.get("type", "") == "speak":
                utterance = msg["data"]["utterance"]
                logger.info("Output: " + utterance)
            if msg.get("type", "") == "server.complete_intent_failure":
                logger.error("Output: complete_intent_failure")
        else:
            pass

    def onClose(self, wasClean, code, reason):
        logger.info("WebSocket connection closed: {0}".format(reason))
        self.factory.client = None
        self.factory.status = "disconnected"

    # cli input thread
    def get_cli_input(self):
        while True:
            line = input("Input: ")
            msg = {"data": {"utterances": [line], "lang": "en-us"},
                   "type": "recognizer_loop:utterance",
                   "context": {"source": self.peer, "destinatary":
                       "https_server", "platform": platform}}
            msg = bytes(json.dumps(msg), "utf-8")
            self.sendMessage(msg, False)


class JarbasCliClientFactory(WebSocketClientFactory, ReconnectingClientFactory):
    protocol = JarbasCliClientProtocol

    def __init__(self, *args, **kwargs):
        super(JarbasCliClientFactory, self).__init__(*args, **kwargs)
        self.status = "disconnected"
        self.client = None

    # websocket handlers
    def clientConnectionFailed(self, connector, reason):
        logger.info("Client connection failed: " + str(reason) + " .. retrying ..")
        self.status = "disconnected"
        self.retry(connector)

    def clientConnectionLost(self, connector, reason):
        logger.info("Client connection lost: " + str(reason) + " .. retrying ..")
        self.status = "disconnected"
        self.retry(connector)


def connect_to_hivemind(host="127.0.0.1",
                        port=5678, name="Standalone Cli Client",
                        api="test_key", useragent=platform):
    authorization = name + ":" + api
    usernamePasswordDecoded = bytes(authorization, "utf-8")
    api = base64.b64encode(usernamePasswordDecoded)
    headers = {'authorization': api}
    address = u"wss://" + host + u":" + str(port)
    factory = JarbasCliClientFactory(address, headers=headers,
                                     useragent=useragent)
    factory.protocol = JarbasCliClientProtocol
    contextFactory = ssl.ClientContextFactory()
    reactor.connectSSL(host, port, factory, contextFactory)
    reactor.run()


if __name__ == '__main__':
    connect_to_hivemind()



import base64
import json
import logging
import socket
import string
import sys
from threading import Thread

from autobahn.twisted.websocket import WebSocketClientFactory, \
    WebSocketClientProtocol
from twisted.internet import reactor, ssl
from twisted.internet.protocol import ReconnectingClientFactory

platform = "JarbasTwitchBridgev0.1"
logger = logging.getLogger(platform)
logger.addHandler(logging.StreamHandler(sys.stdout))
logger.setLevel("INFO")


class JarbasTwitchClientProtocol(WebSocketClientProtocol):
    # Set all the variables necessary to connect to Twitch IRC
    HOST = "irc.twitch.tv"
    NICK = "jarbas_le_bot"
    PORT = 6667
    PASS = "oauth:GET_YOURS"
    CHANNELNAME = "jarbasai"

    def onConnect(self, response):
        logger.info("Server connected: {0}".format(response.peer))
        self.factory.client = self
        self.factory.status = "connected"

    def onOpen(self):
        logger.info("WebSocket connection open. ")
        self.NICK = self.factory.username
        self.PASS = self.factory.oauth
        self.connect_to_twitch()
        self.input_loop = Thread(target=self.twitch_loop)
        self.input_loop.setDaemon(True)
        self.input_loop.start()

    def connect_to_twitch(self):
        # Connecting to Twitch IRC by passing credentials and joining a certain channel
        self.s = socket.socket()
        self.s.connect((self.HOST, self.PORT))
        self.s.send("PASS " + self.PASS + "\r\n")
        self.s.send("NICK " + self.NICK + "\r\n")
        self.s.send("JOIN #" + self.CHANNELNAME + " \r\n")

    # Method for sending a message
    def twitch_send(self, message):
        self.s.send("PRIVMSG #" + self.CHANNELNAME + " :" + message + "\r\n")

    def onMessage(self, payload, isBinary):
        if not isBinary:
            payload = payload.decode("utf-8")
            msg = json.loads(payload)
            if msg.get("type", "") == "speak":
                utterance = msg["data"]["utterance"]
                user = msg.get("context", {}).get("user")
                logger.info("Output: " + utterance)
                if user:
                    utterance += " " + user
                self.twitch_send(utterance)
            if msg.get("type", "") == "server.complete_intent_failure":
                logger.error("Output: complete_intent_failure")
                self.twitch_send("does not compute")
        else:
            pass

    def onClose(self, wasClean, code, reason):
        logger.info("WebSocket connection closed: {0}".format(reason))
        self.factory.client = None
        self.factory.status = "disconnected"

    def twitch_loop(self):

        readbuffer = ""
        MODT = False

        while True:
            readbuffer = readbuffer + self.s.recv(1024)
            temp = string.split(readbuffer, "\n")
            readbuffer = temp.pop()

            for line in temp:
                # Checks whether the message is PING because its a method of Twitch to check if you're afk
                if (line[0] == "PING"):
                    self.s.send("PONG %s\r\n" % line[1])
                else:
                    # Splits the given string so we can work with it better
                    parts = string.split(line, ":")

                    if "QUIT" not in parts[1] and "JOIN" not in parts[1] and "PART" not in parts[1]:
                        try:
                            # Sets the message variable to the actual message sent
                            message = parts[2][:len(parts[2]) - 1]
                        except:
                            message = ""
                        # Sets the username variable to the actual username
                        usernamesplit = string.split(parts[1], "!")
                        username = usernamesplit[0]
                        print("twitch message: ", message)
                        if "@" + self.NICK in message:
                            message = message.replace("@" + self.NICK, "")
                            # Only works after twitch is done announcing stuff (MODT = Message of the day)
                            if MODT:
                                msg = {"data": {"utterances": [message], "lang": "en-us"},
                                       "type": "recognizer_loop:utterance",
                                       "context": {"source": "twitch", "destinatary":
                                           "https_server", "platform": platform, "user": username}}
                                msg = json.dumps(msg)
                                self.sendMessage(bytes(msg, "utf-8"), False)

                        for l in parts:
                            if "End of /NAMES list" in l:
                                MODT = True


class JarbasTwitchClientFactory(WebSocketClientFactory, ReconnectingClientFactory):
    protocol = JarbasTwitchClientProtocol
    oauth = ""
    channel = ""
    username = "Jarbas_BOT"

    def __init__(self, *args, **kwargs):
        super(JarbasTwitchClientFactory, self).__init__(*args, **kwargs)
        self.status = "disconnected"
        self.client = None

    # websocket handlers
    def clientConnectionFailed(self, connector, reason):
        logger.info("Client connection failed: " + str(reason) + " .. retrying ..")
        self.status = "disconnected"
        self.retry(connector)

    def clientConnectionLost(self, connector, reason):
        logger.info("Client connection lost: " + str(reason) + " .. retrying ..")
        self.status = "disconnected"
        self.retry(connector)


def connect_to_twitch(oauth, channel, username="Jarbas_BOT", host="127.0.0.1",
                        port=5678, name="Standalone Twitch Bridge",
                      api="test_key", useragent=platform):
    authorization = name + ":" + api
    usernamePasswordDecoded = bytes(authorization, "utf-8")
    api = base64.b64encode(usernamePasswordDecoded)
    headers = {'authorization': api}
    address = u"wss://" + host + u":" + str(port)
    factory = JarbasTwitchClientFactory(address, headers=headers,
                                        useragent=useragent)
    factory.protocol = JarbasTwitchClientProtocol
    contextFactory = ssl.ClientContextFactory()
    reactor.connectSSL(host, port, factory, contextFactory)
    reactor.run()


if __name__ == '__main__':
    # TODO parse args
    connect_to_twitch("", "")





import os

from setuptools import setup


def package_files(directory):
    paths = []
    for (path, directories, filenames) in os.walk(directory):
        for filename in filenames:
            paths.append(os.path.join('..', path, filename))
    return paths


extra_files = package_files('jarbas_hive_mind')
setup(
    name='jarbas_hive_mind',
    version='0.2',
    packages=['jarbas_hive_mind', 'jarbas_hive_mind.minds',
              'jarbas_hive_mind.nodes', 'jarbas_hive_mind.nodes.flask',
              'jarbas_hive_mind.nodes.webchat', 'jarbas_hive_mind.utils',
              'jarbas_hive_mind.drones',
              'jarbas_hive_mind.bridges', 'jarbas_hive_mind.database',
              'jarbas_hive_mind.terminals',
              'jarbas_hive_mind.terminals.speech',
              'jarbas_hive_mind.terminals.speech.stt',
              'jarbas_hive_mind.terminals.speech.recognizer',
              'jarbas_hive_mind.terminals.webchat'],
    url='https://github.com/JarbasAl/hive_mind',
    license='MIT',
    package_data={'': extra_files},
    include_package_data=True,
    install_requires=['pyopenssl',
                      'service_identity',
                      'flask',
                      'flask-sslify',
                      'yagmail',
                      'autobahn',
                      'twisted',
                      'sqlalchemy',
                      'tornado==4.5.3',
                      'SpeechRecognition',
                      'responsivevoice',
                      'hclib',
                      'fbchat',
                      'remi'],
    author='jarbasAI',
    author_email='jarbasai@mailfence.com',
    description='Telepathy for mycroft core'
)



from mycroft.skills.intent_service import IntentService, normalize, LOG, open_intent_envelope


class MicroIntentService(IntentService):

    def __init__(self, ws):
        self.intent_map = {}
        self.skills_map = {}
        self.vocab_map = {}
        IntentService.__init__(self, ws)
        self.bus.on("mycroft.skills.loaded", self.handle_skill_load)
        self.bus.on("mycroft.skills.shutdown", self.handle_skill_shutdown)

    def handle_utterance(self, message):
        pass

    def handle_register_vocab(self, message):
        start_concept = message.data.get('start')
        end_concept = message.data.get('end')
        regex_str = message.data.get('regex')
        alias_of = message.data.get('alias_of')
        if regex_str:
            self.engine.register_regex_entity(regex_str)
        else:
            if start_concept:
                self.vocab_map[start_concept] = end_concept
            self.engine.register_entity(
                start_concept, end_concept, alias_of=alias_of)

    def handle_register_intent(self, message):
        intent = open_intent_envelope(message)
        self.engine.register_intent_parser(intent)
        skill_id, intent = message.data.get("name", "None:None").split(":")
        LOG.info("Registered: " + intent)
        if skill_id not in self.intent_map.keys():
            self.intent_map[skill_id] = []
        self.intent_map[skill_id].append(intent)

    def handle_detach_intent(self, message):
        intent_name = message.data.get('intent_name')
        new_parsers = [
            p for p in self.engine.intent_parsers if p.name != intent_name]
        self.engine.intent_parsers = new_parsers
        skill_id, intent = intent_name.split(":")
        self.intent_map[skill_id].pop(intent)

    def handle_detach_skill(self, message):
        skill_id = message.data.get('skill_id')
        new_parsers = [
            p for p in self.engine.intent_parsers if
            not p.name.startswith(skill_id)]
        self.engine.intent_parsers = new_parsers
        self.intent_map.pop(skill_id)

    def handle_skill_shutdown(self, message):
        name = message.data.get("name")
        id = message.data.get("id")
        self.skills_map[id] = name

    def handle_skill_load(self, message):
        id = message.data.get("id")
        self.skills_map.pop(id)

    def get_skills_map(self, lang="en-us"):
        return self.skills_map

    def get_intent_map(self, lang="en-us"):
        return self.intent_map

    def get_vocab_map(self, lang="en-us"):
        return self.vocab_map

    def get_intent(self, utterance, lang="en-us"):
        best_intent = None
        try:
            # normalize() changes "it's a boy" to "it is boy", etc.
            best_intent = next(self.engine.determine_intent(
                normalize(utterance, lang), 100,
                include_tags=True,
                context_manager=self.context_manager))
            # TODO - Should Adapt handle this?
            best_intent['utterance'] = utterance
        except Exception as e:
            LOG.exception(e)

        if best_intent and best_intent.get('confidence', 0.0) > 0.0:
            return best_intent
        else:
            return None

    def shutdown(self):
        self.bus.remove("mycroft.skills.loaded", self.handle_skill_load)
        self.bus.remove("mycroft.skills.shutdown", self.handle_skill_shutdown)



from contextlib import contextmanager
from os import makedirs
from os.path import join, expanduser, isdir

from sqlalchemy import Column, Text, String, Integer, create_engine, Boolean
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm import sessionmaker

from jarbas_hive_mind.database import Base


@contextmanager
def session_scope(db):
    """Provide a transactional scope around a series of operations."""
    Session = sessionmaker(bind=db)
    session = Session()
    try:
        yield session
        session.commit()
    except:
        session.rollback()
        raise
    finally:
        session.close()


class Client(Base):
    __tablename__ = "clients"
    id = Column(Integer, primary_key=True)
    description = Column(Text)
    api_key = Column(String)
    name = Column(String)
    mail = Column(String)
    last_seen = Column(Integer, default=0)
    is_admin = Column(Boolean, default=False)


class ClientDatabase(object):
    db_dir = expanduser("~/jarbas_hivemind/database")
    if not isdir(db_dir):
        makedirs(db_dir)
    default_db = "sqlite:///" + join(db_dir, "clients.db")

    def __init__(self, path=None, debug=False):
        if path is None:
            try:
                from mycroft.configuration.config import Configuration
                path = Configuration.get().get("hivemind", {})\
                    .get("sql_client_db", self.default_db)
            except ImportError:
                path = self.default_db

        self.db = create_engine(path)
        self.db.echo = debug

        Base.metadata.create_all(self.db)

    def update_timestamp(self, api, timestamp):
        with session_scope(self.db) as session:
            user = self.get_client_by_api_key(api)
            if not user:
                return False
            user.last_seen = timestamp
            return True

    def delete_client(self, api):
        with session_scope(self.db) as session:
            user = self.get_client_by_api_key(api)
            if user:
                session.delete(user)
                return True
            return False

    def change_api(self, user_name, new_key):
        with session_scope(self.db) as session:
            user = self.get_client_by_name(user_name)
            if not user:
                return False
            user.api_key = new_key

    def change_name(self, new_name, key):
        with session_scope(self.db) as session:
            user = self.get_client_by_api_key(key)
            if not user:
                return False
            user.name = new_name

    def get_client_by_api_key(self, api_key):
        with session_scope(self.db) as session:
            return session.query(Client).filter_by(api_key=api_key).first()

    def get_client_by_name(self, name):
        with session_scope(self.db) as session:
            return session.query(Client).filter_by(name=name).first()

    def add_client(self, name=None, mail=None, api="", admin=False):
        with session_scope(self.db) as session:
            user = Client(api_key=api, name=name, mail=mail,
                      id=self.total_clients() + 1, is_admin=admin)

            session.add(user)

    def total_clients(self):
        with session_scope(self.db) as session:
            return session.query(Client).count()

    def commit(self, handler):
        Session = sessionmaker(bind=self.db)
        session = Session()
        try:
            handler(session)
            session.commit()
        except IntegrityError:
            session.rollback()
            raise
        finally:
            session.close()


if __name__ == "__main__":
    db = ClientDatabase(debug=True)
    name = "jarbas"
    mail = "jarbasaai@mailfence.com"
    api = "test_key"
    db.add_client(name, mail, api, admin=False)



import json
import os
import ssl
import time
from functools import wraps

from flask import Flask, make_response, request, Response
from flask_sslify import SSLify

from jarbas_hive_mind.database.client import ClientDatabase
from jarbas_hive_mind.nodes import gen_api


def root_dir():
    """ Returns root directory for this project """
    return os.path.dirname(os.path.dirname(os.path.realpath(__file__ + '/.')))


def nice_json(arg):
    response = make_response(json.dumps(arg, sort_keys = True, indent=4))
    response.headers['Content-type'] = "application/json"
    return response

app = Flask(__name__)
sslify = SSLify(app)
port = 5678


users = ClientDatabase()


def add_response_headers(headers=None):
    """This decorator adds the headers passed in to the response"""
    headers = headers or {}

    def decorator(f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            resp = make_response(f(*args, **kwargs))
            h = resp.headers
            for header, value in headers.items():
                h[header] = value
            return resp

        return decorated_function

    return decorator


def noindex(f):
    """This decorator passes X-Robots-Tag: noindex"""
    return add_response_headers({'X-Robots-Tag': 'noindex'})(f)


def donation(f):
    """This decorator passes btc request """
    return add_response_headers({'BTC':
                                     '1aeuaAijzwK4Jk2ixomRkqjF6Q3JxXp9Q',
                                 "Patreon": "patreon.com/jarbasAI",
                                 "Paypal": "paypal.me/jarbasAI"})(
        f)


def check_auth(api_key):
    """This function is called to check if a api key is valid."""
    user = users.get_client_by_api_key(api_key)
    if not user:
        return False
    users.update_timestamp(api_key, time.time())
    return True


def check_admin_auth(api_key):
    """This function is called to check if a admin api key is valid."""
    user = users.get_client_by_api_key(api_key)
    if not user:
        return False
    if not user.is_admin:
        return False
    users.update_timestamp(api_key, time.time())
    return True


def authenticate():
    """Sends a 401 response that enables basic auth"""
    return Response(
        'Could not verify your access level for that URL.\n'
        'You have to login with proper credentials', 401,
        {'WWW-Authenticate': 'Basic realm="Api Key Required"'})


def requires_auth(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        auth = request.headers.get('Authorization', '')
        if not auth or not check_auth(auth):
            return authenticate()
        return f(*args, **kwargs)

    return decorated


def requires_admin(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        auth = request.headers.get('Authorization', '')
        if not auth or not check_admin_auth(auth):
            print("not admin")
            return authenticate()
        return f(*args, **kwargs)

    return decorated


@app.route("/", methods=['GET'])
@noindex
@donation
def hello():
    return nice_json({
        "uri": "/",
        "welcome to jarbas microservices": {
            "under": "construction"
        }
    })


@app.route("/revoke_api/<api>", methods=['PUT'])
@noindex
@donation
@requires_admin
def revoke_api(api):
    if users.delete_client(api):
        result = {"removed": True}
    else:
        result = {"removed": False, "error": "does not exist"}
    return nice_json(
        result
    )


@app.route("/new_user/<api>/<mail>/<name>", methods=['PUT'])
@noindex
@donation
@requires_admin
def add_user(api, mail, name):
    result = {"success": False}
    if users.add_client(name, mail, api):
        result = {"success": True}
    return nice_json(
        result
    )


@app.route("/get_api", methods=['GET'])
@noindex
@donation
@requires_admin
def new_api():
    api = gen_api(save=False)
    return nice_json(
        {"api": api}
    )


def start(app, port=6669):
    cert = "{}/certs/default.crt".format(root_dir())
    key = "{}/certs/default.key".format(root_dir())
    context = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
    context.load_cert_chain(cert, key)
    app.run(host="0.0.0.0", port=port, debug=False, ssl_context=context)


if __name__ == "__main__":
    global app
    start(app, port)
